
MAX_WORDS_IN_SENTENCE = 20

MAX_SENTENCES_IN_PARAGRAPH = 6

WORDS_ONE_READS_PER_MINUTE = 200

VAGUE_WORDS = [
    "approach", "approaches", 'assumption', "assumptions", 'concept', "concepts", 'condition',
    "conditions", 'context', 'contexts', 'framework', 'frameworks', 'issue', 'issues', 'process',
    'processes', 'range', 'role', 'roles', 'strategy', 'strategies', 'tendency', 'tendencies',
    'variable', 'variables', 'perspective', 'perspectives',  'accrual', 'derivative', 'derivatives',
    'fair value', 'portfolio', 'portfolios', 'audit', 'poverty', 'evaluation', 'evaluations', 'management',
    'monitoring', 'effectiveness', 'performance', 'competitiveness', 'reform', 'assistance', 'growth',
    'effort', 'capacity', 'transparency', 'effectiveness', 'progress', 'stability', 'protection', 'access',
    'implementation', 'sustainable', 'stuff', 'really', 'etc', 'many', 'usually',
    'most', 'somehow', 'somewhat', 'somewhat like', 'soon', 'a few'

]

INTENSIFIERS = ['very', 'highly', 'extremely']

COMPULSIVE_HEDGERS = [
    'apparently', 'almost', 'fairly', 'nearly', 'partially', 'predominantly', 'presumably', 'rather', 'relative',
    'seemingly', 'sort of', 'kind of', 'a bit', 'a little', 'maybe', 'some', 'partly', 'perhaps', 'any', 'probably',
    'a touch', 'sometimes', 'mostly', 'possibly', 'might', 'a tad', 'hardly', 'seem'
]






from cmdline_printer import ArticlePrinter
from analyzer import Article


########## Following will be removed from the Final code ##############

editorial_1_july = "/Users/waqas/code/homer/experiment/newspapers/editorial_1_july_19_br.txt"
editorial_2_july = "/Users/waqas/code/homer/experiment/newspapers/editorial_2nd_july_19.txt"
editorial_29_june = "/Users/waqas/code/homer/experiment/newspapers/editorial_29th_july_br.txt"
editorial_25_june = "/Users/waqas/code/homer/experiment/newspapers/editorial_june_25_br.txt"
editorial_26_june = "/Users/waqas/code/homer/experiment/newspapers/editorial_june_26_br.txt"
editorial_28_jne = "/Users/waqas/code/homer/experiment/newspapers/editorial_june_28_br.txt"


oped_9_feb_hafiz_pasha = "/Users/waqas/code/homer/experiment/newspapers/more_stagflation_9_feb_2019_br_hafiz_pasha"
oped_2_july_hafiz_pasha = "/Users/waqas/code/homer/experiment/newspapers/stab_effort_2_july_19_hafiz_pasha_br.txt"
oped_1_july_anjum = "/Users/waqas/code/homer/experiment/newspapers/tax_man_anjum_ibrahim_1_july_19_br.txt"
oped_1_july_andeel = "/Users/waqas/code/homer/experiment/newspapers/changing_perception_andeel_1_july_19_br.txt"

oped_dawn_5_july_sakib = '/Users/waqas/code/homer/experiment/newspapers/dawn_oped_sakib_sherani_5_july.txt'
oped_dawn_5_july_jihad_azour = '/Users/waqas/code/homer/experiment/newspapers/dawn_oped_jihad_azour_5_july.txt'
shabaz_4_july = '/Users/waqas/code/homer/experiment/newspapers/shabaz_july4_ecc_rejects.txt'

# econ survey
fiscal_dev = '/Users/waqas/code/homer/experiment/economic_survey_pk_2018_19/fiscal_development.txt'
inflation = '/Users/waqas/code/homer/experiment/economic_survey_pk_2018_19/inflation.txt'
manufacturing_and_mining = '/Users/waqas/code/homer/experiment/economic_survey_pk_2018_19/manufacturing_and_mining.txt'
money_and_credit = '/Users/waqas/code/homer/experiment/economic_survey_pk_2018_19/money_and_credit.txt'
overview_of_econ = '/Users/waqas/code/homer/experiment/economic_survey_pk_2018_19/overview_of_economy.txt'
oped = "/Users/waqas/code/homer/experiment/oped.txt"
roadmap = '/Users/waqas/code/homer/experiment/economic_survey_pk_2018_19/roadmap_for_stability_and_growth_mof.txt'
encoding = '/Users/waqas/code/homer/experiment/encoding/CDS.txt'
article = ArticlePrinter(Article('', '', open(oped).read()))
# article = Article('name', 'author', open('/Users/waqas/PycharmProjects/sense_of_style/experiment/economic_survey_pk_2018_19/overview_of_economy.txt').read())
article.print_article_stats()
# # print(article.words_with_most_syllables())
article.print_paragraph_stats()



import textstat


class FleschReading(object):
    """
    https://en.wikipedia.org/wiki/Flesch%E2%80%93Kincaid_readability_tests#Flesch_reading_ease
    """
    def __init__(self, text):
        self.score = textstat.flesch_reading_ease(text)
        self.difficulty_threshold = 60

    def is_difficult(self):
        return True if self.score < self.difficulty_threshold else False

    def grade(self):
        if self.score < 30:
            return 'Very difficult'
        elif 30 <= self.score < 50:
            return 'Difficult'
        elif 50 <= self.score < 60:
            return 'Fairly difficult'
        elif 60 <= self.score < 70:
            return 'Plain English'
        elif 70 <= self.score < 80:
            return 'Fairly easy'
        elif 80 <= self.score < 90:
            return 'Easy'
        elif 90 <= self.score <= 100:
            return 'Very easy'


class DaleChall(object):
    """
    https://en.wikipedia.org/wiki/Dale%E2%80%93Chall_readability_formula
    """

    def __init__(self, text):
        self.score = round(textstat.dale_chall_readability_score(text), 1)

    def grade(self):
        if self.score <= 4.9:
            return 'Average 4th grade student or lower'
        elif 5.0 <= self.score <= 5.9:
            return 'Average 5th or 6th grade student'
        elif 6.0 <= self.score <= 6.9:
            return 'Average 7th or 8th grade student'
        elif 7.0 <= self.score <= 7.9:
            return 'Average 9th or 10th grade student'
        elif 8.0 <= self.score <= 8.9:
            return 'Average 11th or 12th grade student'
        elif self.score >= 9.0:
            return 'Average 13th or 15th grade student'



"""
A command line utility to use Homer. Example usage:

> python homer_cmd.py --help

Above shows help. Following will display stats on the command line:

> python homer_cmd.py --name sdf --author dede --file_path=/correct/path/to/file.txt

"""
import os
import click
from homer.analyzer import Article
from homer.cmdline_printer import ArticlePrinter

@click.command()
@click.option('--name', help='Article name, can be an empty string.')
@click.option('--author', help='Author name, can be an empty string.')
@click.option('--file_path', required=True, type=click.Path(exists=True))
def homer_cmd(name, author, file_path):
    file_path = os.path.abspath(file_path)
    printer = ArticlePrinter(Article(name, author, open(file_path, mode='r', encoding='utf-8').read()))
    printer.print_article_stats()
    printer.print_paragraph_stats()


if __name__ == "__main__":
    homer_cmd()





from colorclass import Color
from terminaltables import SingleTable


class ArticlePrinter(object):
    """Class which can be used to print Article stats on the command line"""

    def __init__(self, article_obj):
        self.article = article_obj

    def print_article_stats(self):
        """This method is called to present overall article stats on a command line."""
        table_data = [
            [Color('{autocyan}Overall Stats{/autocyan}')],
            ['Reading time', str(self.article.reading_time) + ' mins'],
            ['Flesch Reading Ease', self.article.get_flesch_reading_score()],
            ['Dale Chall Readability Score', self.article.get_dale_chall_reading_score()],
            ['Paragraphs', self.article.total_paragraphs],
            ['Avg sentences per paragraph', self.article.avg_sentences_per_para],
            ['Total sentences in longest paragraph', self.article.len_of_longest_paragraph],
            ['Sentences', self.article.total_sentences],
            ['Avg words per sentence', self.article.avg_words_per_sentence],
            ['Longest sentence', "%s..." % str(self.article.longest_sentence)[0:30]],
            ['Words in longest sentence', self.article.len_of_longest_sentence],
            ['Words', self.article.total_words],
            ['"and" frequency"', self.article.get_and_frequency()],
            ['Compulsive Hedgers', len(self.article.get_compulsive_hedgers())],
            ['Intensifiers', len(self.article.get_intensifiers())],
            ['Vague words', len(self.article.get_vague_words())],
        ]
        table_instance = SingleTable(table_data)
        table_instance.inner_heading_row_border = True
        table_instance.inner_row_border = True
        table_instance.justify_columns = {0: 'left', 1: 'center'}
        print(table_instance.table)
        self.print_detail()

    def print_paragraph_stats(self):
        """This method, along with print_article_stats(), can be called to present paragraph stats on a command line.
        Ideally first call print_article_stats() and then this method.
        It shows sentence, average words per sentence, longest sentence, and readability scores (Flesch reading ease and
        Dale Chall readability scores) of paragraphs.
        """

        sentence_tag = Color('{blue}sentences{/blue}')
        word_tag = Color('{blue}words{/blue}')
        avg_word_tag = Color('{blue}Avg words per sentence{/blue}')
        long_tag = Color('{red}longest{/red}')
        table_data = [
            [Color('{autocyan}Paragraph Stats{/autocyan}')],
            ['Paragraph #', '']
        ]
        for item, para in enumerate(self.article.paragraphs):
            sentences = Color('{red}%s{/red}' % str(len(para))) if len(para) > 5 else str(len(para))
            avg_words_per_sentence = Color(
                '{red}%s{/red}' % str(para.avg_words_per_sentence)) if para.avg_words_per_sentence > 25 else str(
                para.avg_words_per_sentence)
            table_data.append([item + 1,
                               '{sentences} {sent_tag}. {words} {word_tag}. {avg_words} {avg_word_tag}. '
                               '"{longest_sent}..." is the {long_tag} sentence.'.format(
                                   sentences=sentences, sent_tag=sentence_tag, words=para.total_words,
                                   word_tag=word_tag, avg_words=avg_words_per_sentence, avg_word_tag=avg_word_tag,
                                   longest_sent=str(para.longest_sentence)[0:10], long_tag=long_tag
                               )])
            table_data.append(["", "Flesh Reading score={flesch_reading}, Dale Chall Readability= {dale_chall}".format(
                flesch_reading=para.get_flesch_reading_score(), dale_chall=para.get_dale_chall_reading_score()
            )])

        table_instance = SingleTable(table_data)
        table_instance.inner_heading_row_border = True
        table_instance.inner_row_border = True
        table_instance.justify_columns = {0: 'center', 1: 'left'}
        print(table_instance.table)

    def _print_detail_of(self, words_list, heading, display_count=True):
        words = [str(word) for word in words_list]
        if len(words) >= 1:
            words_unique_list = list(set(words))
            format_str = "{word} ({count})"
            if display_count:
                msg = '{red} **- %s: %s {/red}\r\n' % (heading, ', '.join(format_str.format(word=str(word),
                                                           count=words.count(word)) for word in words_unique_list))
            else:
                msg = '{red} **- %s: %s {/red}\r\n' % (heading, ', '.join(word for word in words_unique_list))
            print(Color(msg))

    def print_detail(self):
        self._print_detail_of(self.article.get_compulsive_hedgers(), "Compulsive Hedgers", display_count=False)
        self._print_detail_of(self.article.get_intensifiers(), "Intensifiers", display_count=False)
        self._print_detail_of(self.article.get_vague_words(), "Vague words")
        self._print_detail_of(self.article.ten_words_with_most_syllables(), "10 words with most syllables", display_count=False)
        self._print_detail_of(self.article.get_n_most_repeated_words(20), "20 most repeated words", display_count=False)




__author__ = 'Waqas Younas'

"""
Homer: A text analyzer.
"""

# Always prefer setuptools over distutils
import os
from setuptools import setup
from distutils.command.install import install as _install
# To use a consistent encoding
from codecs import open

this_directory = os.path.abspath(os.path.dirname(__file__))


class install(_install):
    def run(self):
        _install.run(self)
        import nltk
        nltk.download('punkt')
        # after punkt is install, let's install other packages one by one
        nltk.download('averaged_perceptron_tagger')
        nltk.download('cmudict')
        nltk.download('stopwords')


with open(os.path.join(this_directory, 'README.rst'), encoding='utf-8') as f:
    readme = f.read()

setup(
    name='homer_text',
    version='0.4.1',
    description='Homer, a text analyser in Python, can help make your text more clear, simple and useful for your readers.',
    long_description=readme,
    long_description_content_type='text/markdown',
    packages=['homer'],
    packages_dir={'homer': 'homer'},
    include_package_data=True,
    # The project's main homepage.
    url='https://github.com/wyounas/homer',

    # Author details
    author='Waqas Younas',
    author_email='waqas.younas@gmail.com',

    # Choose your license
    license='MIT',

    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 3 - Alpha',

        # Indicate who your project is intended for
        'Intended Audience :: Developers',
        'Topic :: Text Processing :: Linguistic',

        # Pick your license as you wish (should match "license" above)
        'License :: OSI Approved :: MIT License',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],

    # What does your project relate to?
    keywords='text analyzer',
    project_urls={
        "Bug Tracker": "https://github.com/wyounas/homer/issues",
        "Documentation": "https://github.com/wyounas/homer",
        "Source Code": "https://github.com/wyounas/homer",
    },

    # You can just specify the packages manually here if your project is
    # simple. Or you can use find_packages().
    # packages=find_packages(exclude=['contrib', 'docs', 'tests']),

    # Alternatively, if you want to distribute just a my_module.py, uncomment
    # this:
    #   py_modules=["my_module"],

    # List run-time dependencies here.  These will be installed by pip when
    # your project is installed. For an analysis of "install_requires" vs pip's
    # requirements files see:
    # https://packaging.python.org/en/latest/requirements.html
    install_requires=[
        'Click==7.0',
        'colorclass==2.2.0',
        'nltk==3.4.1',
        'Pyphen==0.9.5',
        'repoze.lru==0.7',
        'six==1.12.0',
        'terminaltables==3.1.0',
        'textstat==0.5.6',
    ],
    setup_requires=['nltk==3.4.1'],
    cmdclass={'install': install}
)





"""

                 _   _
                | | | | ___  _ __ ___   ___ _ __
                | |_| |/ _ \| '_ ` _ \ / _ \ '__|
                |  _  | (_) | | | | | |  __/ |
                |_| |_|\___/|_| |_| |_|\___|_|


Homer can help make your text more clear, simple and useful for the reader.

Homer analyzes english text (e.g. a blog post or an essay). Gives reading time, readability scores, paragraph and
sentence level stats.

Author: Waqas Younas
Email: waqas.younas@gmail.com
"""
import operator
import itertools
import nltk
from nltk.corpus import cmudict
from functools import lru_cache
from homer.utils import FleschReading, DaleChall
from homer.constants import INTENSIFIERS, COMPULSIVE_HEDGERS, VAGUE_WORDS, MAX_WORDS_IN_SENTENCE,\
    MAX_SENTENCES_IN_PARAGRAPH, WORDS_ONE_READS_PER_MINUTE


class Word(object):
    """
    An abstraction of a 'word'. It determines whether a word is a compulsive hedger, an intensifier or a vague word.

    The idea to look for compulsive hedgers, intensifiers and vague words came from the following works:
    - Steven Pinker's book `The Sense of Style: The Thinking Person's Guide to Writing in the 21st Century`.
    - https://litlab.stanford.edu/LiteraryLabPamphlet9.pdf
    """

    def __init__(self, word, intensifiers=INTENSIFIERS, compulsive_hedgers=COMPULSIVE_HEDGERS,
                 vague_words=VAGUE_WORDS):
        self.syllables = None
        self.word = word.strip().lower()
        self.intensifiers = intensifiers
        self.compulsive_hedgers = compulsive_hedgers
        self.vague_words = vague_words

    def is_and(self):
            word_and = 'and'
            return self.word == word_and

    def is_compulsive_hedger(self):
        return self.word in self.compulsive_hedgers

    def is_intensifier(self):
        return self.word in self.intensifiers

    def is_vague_word(self):
        return self.word in self.vague_words

    def __repr__(self):
        return 'Word(%r)' % self.word

    def __str__(self):
        return self.word


@lru_cache(maxsize=256)
def get_word(word):
    return Word(word)


class Sentence(object):
    """
    An abstraction that represents a sentence and gives various stats.
    """
    def __init__(self, sentence):
        self.sentence = sentence
        words = nltk.word_tokenize(sentence)
        words = [word for word in words if word.isalpha() or word.isdigit()]
        self._words = [get_word(word) for word in words]

    @property
    def words(self):
        return self._words

    @property
    def total_and_words(self):
        return len([word for word in self.words if word.is_and()])

    def is_long(self):
        return len(self) > MAX_WORDS_IN_SENTENCE

    def get_compulsive_hedgers(self):
        return (word for word in self.words if word.is_compulsive_hedger())

    def get_intensifiers(self):
        return (word for word in self.words if word.is_intensifier())

    def get_vague_words(self):
        return (word for word in self.words if word.is_vague_word())

    def __len__(self):
        return len(self.words)

    def __lt__(self, other):
        return len(self) < len(other)

    def __gt__(self, other):
        return len(self) > len(other)

    def __eq__(self, other):
        return len(self) == len(other)

    def __repr__(self):
        return 'Sentence(%r)' % self.sentence

    def __str__(self):
        return self.sentence


class Paragraph(object):
    """
    Represents a paragraph. Finds compulsive hedgers, intensifiers, vague words, readability scores of a paragraph and
    various other stats.
    """

    def __init__(self, paragraph):
        paragraph = paragraph.replace('—', ' ')
        self.paragraph = paragraph
        self.tokenized_sentences = nltk.sent_tokenize(paragraph)
        self._sentences = [Sentence(sentence) for sentence in self.tokenized_sentences]


    @property
    def sentences(self):
        return self._sentences

    @property
    def longest_sentence(self):
        return max(self._sentences, key=lambda sentence: len(sentence))

    @property
    def total_words(self):
        return sum([len(sentence) for sentence in self.sentences])

    @property
    def total_and_words(self):
        return sum([sentence.total_and_words for sentence in self.sentences if sentence.total_and_words])

    @property
    def avg_words_per_sentence(self):
        round_to_two_digits = 2
        return round(self.total_words / len(self), round_to_two_digits)

    def is_long(self):
        return len(self) >= MAX_SENTENCES_IN_PARAGRAPH

    def get_flesch_reading_score(self):
        return FleschReading(self.paragraph).grade()

    def get_dale_chall_reading_score(self):
        return DaleChall(self.paragraph).grade()

    def get_intensifiers(self):
        return list(itertools.chain(*[sentence.get_intensifiers() for sentence in self.sentences
                                                   if sentence.get_intensifiers()]))

    def get_compulsive_hedgers(self):
        return list(itertools.chain(*[sentence.get_compulsive_hedgers() for sentence in self.sentences
                                                         if sentence.get_compulsive_hedgers()]))

    def get_vague_words(self):
        return list(itertools.chain(*[sentence.get_vague_words() for sentence in self.sentences
                                                  if sentence.get_vague_words()]))

    def __len__(self):
        return len(self.sentences)

    def __lt__(self, other):
        return len(self) < len(other)

    def __gt__(self, other):
        return len(self) > len(other)

    def __eq__(self, other):
        return len(self) == len(other)

    def __repr__(self):
        return 'Paragraph(%r)' % self.paragraph


class Article(object):
    """
    This represents a block of text, i.e. an essay, a blog or an article.

    Using this, we can retrieve article as well as paragraph-level stats.
    """
    def __init__(self, name, author, text):
        self.name = name
        self.author = author
        # Replacing em dash and en dash
        self.text = text.replace('—', ' ')
        paragraphs = nltk.tokenize.blankline_tokenize(text)
        self._paragraphs = [Paragraph(paragraph) for paragraph in paragraphs]

    @property
    def paragraphs(self):
        return self._paragraphs

    @property
    def len_of_longest_paragraph(self):
        return len(max(self._paragraphs, key=lambda paragraph: len(paragraph)))

    @property
    def longest_sentence(self):
        return max([paragraph.longest_sentence for paragraph in self._paragraphs], key=lambda sentence: len(sentence))

    @property
    def len_of_longest_sentence(self):
        return len(self.longest_sentence)

    @property
    def total_sentences(self):
        return sum([len(paragraph) for paragraph in self.paragraphs])

    @property
    def total_paragraphs(self):
        return len(self.paragraphs)

    @property
    def total_words(self):
        return sum([paragraph.total_words for paragraph in self.paragraphs])

    @property
    def total_and_words(self):
        return sum([paragraph.total_and_words for paragraph in self.paragraphs])

    @property
    def reading_time(self):
        return self.total_words / WORDS_ONE_READS_PER_MINUTE

    @property
    def avg_sentences_per_para(self):
        round_to_two_digits = 2
        return round(self.total_sentences / self.total_paragraphs, round_to_two_digits)

    @property
    def avg_words_per_sentence(self):
        round_to_two_digits = 2
        return round(self.total_words / self.total_sentences, round_to_two_digits)

    def get_paragraphs(self):
        return self.paragraphs

    def get_flesch_reading_score(self):
        return FleschReading(self.text).grade()

    def get_dale_chall_reading_score(self):
        return DaleChall(self.text).grade()

    def is_difficult_to_read(self):
        return FleschReading(self.text).is_difficult()

    def get_intensifiers(self):
        return list(itertools.chain(*(paragraph.get_intensifiers() for paragraph in self.paragraphs
                                                   if paragraph.get_intensifiers())))

    def get_vague_words(self):
        return list(itertools.chain(*(paragraph.get_vague_words() for paragraph in self.paragraphs
                                                  if paragraph.get_vague_words())))

    def get_compulsive_hedgers(self):
        return list(itertools.chain(*(paragraph.get_compulsive_hedgers() for paragraph in self.paragraphs
                                                         if paragraph.get_compulsive_hedgers())))

    def get_and_frequency(self):
        round_to_two_digits = 2
        return str(round(self.total_and_words / self.total_words * 100, round_to_two_digits)) + " %"

    def ten_words_with_most_syllables(self):
        """This gets us 10 words with most syllables in a text"""
        d = cmudict.dict()
        words = nltk.word_tokenize(self.text)
        syllable_data = {}
        for word in words:
            word = word.strip().lower()
            if word not in syllable_data:
                try:
                    syllable_data[word] = [len(list(y for y in x if y[-1].isdigit())) for x in d[word]][0]
                except KeyError:
                    pass  # Perhaps this word is not in the cmudict
        return [item[0] for item in list(reversed(sorted(syllable_data.items(), key=operator.itemgetter(1))))[:10]]

    def get_n_most_repeated_words(self, n):
        """Gets us n most repeated words in the text. """
        words = nltk.word_tokenize(self.text)
        stopwords = nltk.corpus.stopwords.words('english')
        all_words_except_stop = nltk.FreqDist(w.lower() for w in words if w[0].isalpha() and w not in stopwords)
        return [word for word, freq in all_words_except_stop.most_common(n)]
